<?php

namespace Drupal\search_api_postgresql\Service;

use Drupal\search_api_postgresql\Exception\DatabaseConnectionException;
use Drupal\search_api_postgresql\Exception\MemoryExhaustedException;
use Drupal\search_api_postgresql\Exception\VectorIndexCorruptedException;
use Drupal\search_api_postgresql\Exception\ApiKeyExpiredException;
use Drupal\search_api_postgresql\Exception\EmbeddingServiceUnavailableException;
use Drupal\search_api_postgresql\Exception\RateLimitException;
use Drupal\search_api_postgresql\Exception\CacheDegradedException;
use Drupal\search_api_postgresql\Exception\CircuitBreakerException;
use Drupal\Core\Database\Connection;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Config\ConfigFactoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Automated error recovery and healing strategies service.
 */
class ErrorRecoveryService {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Circuit breaker states for various services.
   *
   * @var array
   */
  protected $circuitBreakerStates = [];

  /**
   * Recovery attempt tracking.
   *
   * @var array
   */
  protected $recoveryAttempts = [];

  /**
   * Health check results cache.
   *
   * @var array
   */
  protected $healthCheckCache = [];

  /**
   * Constructs an ErrorRecoveryService.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue factory.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(
    Connection $database,
    CacheBackendInterface $cache,
    QueueFactory $queue_factory,
    ConfigFactoryInterface $config_factory,
    LoggerInterface $logger
  ) {
    $this->database = $database;
    $this->cache = $cache;
    $this->queueFactory = $queue_factory;
    $this->configFactory = $config_factory;
    $this->logger = $logger;
  }

  /**
   * Attempts automatic recovery based on error type.
   *
   * @param \Exception $exception
   *   The exception to recover from.
   * @param array $context
   *   Additional context for recovery.
   *
   * @return bool
   *   TRUE if recovery was attempted, FALSE otherwise.
   */
  public function attemptRecovery(\Exception $exception, array $context = []) {
    $strategy = $this->getRecoveryStrategy($exception);
    $recovery_id = $this->generateRecoveryId($exception, $context);

    // Check if we've exceeded maximum recovery attempts
    if ($this->hasExceededMaxAttempts($recovery_id)) {
      $this->logger->warning('Recovery attempts exceeded for {recovery_id}', [
        'recovery_id' => $recovery_id,
        'exception' => get_class($exception),
      ]);
      return FALSE;
    }

    $this->trackRecoveryAttempt($recovery_id);

    try {
      switch ($strategy) {
        case 'reconnect_database':
          return $this->attemptDatabaseReconnection($context);

        case 'rebuild_vector_index':
          return $this->queueVectorIndexRebuild($context);

        case 'clear_cache_and_retry':
          return $this->clearCacheAndRetry($context);

        case 'reduce_batch_size':
          return $this->reduceBatchSize($context);

        case 'restart_external_service':
          return $this->restartExternalService($context);

        case 'rotate_api_keys':
          return $this->rotateApiKeys($context);

        case 'enable_circuit_breaker':
          return $this->enableCircuitBreaker($context);

        case 'scale_resources':
          return $this->scaleResources($context);

        case 'activate_fallback_mode':
          return $this->activateFallbackMode($context);

        case 'emergency_maintenance':
          return $this->triggerEmergencyMaintenance($context);

        default:
          $this->logger->info('No recovery strategy available for {exception}', [
            'exception' => get_class($exception),
            'strategy' => $strategy,
          ]);
          return FALSE;
      }
    }
    catch (\Exception $recovery_exception) {
      $this->logger->error('Recovery attempt failed: {message}', [
        'message' => $recovery_exception->getMessage(),
        'original_exception' => get_class($exception),
        'recovery_strategy' => $strategy,
      ]);
      return FALSE;
    }
  }

  /**
   * Performs comprehensive health check and proactive error prevention.
   *
   * @return array
   *   Health check results.
   */
  public function performHealthCheck() {
    $cache_key = 'search_api_postgresql:health_check:' . date('Y-m-d-H-i');
    
    if ($cached = $this->cache->get($cache_key)) {
      return $cached->data;
    }

    $checks = [
      'database_connectivity' => $this->checkDatabaseHealth(),
      'memory_usage' => $this->checkMemoryUsage(),
      'disk_space' => $this->checkDiskSpace(),
      'vector_indexes' => $this->checkVectorIndexHealth(),
      'external_services' => $this->checkExternalServices(),
      'cache_performance' => $this->checkCachePerformance(),
      'queue_health' => $this->checkQueueHealth(),
      'api_quotas' => $this->checkApiQuotas(),
    ];

    // Overall health assessment
    $overall_status = $this->assessOverallHealth($checks);
    $checks['overall'] = $overall_status;

    // Proactive recommendations
    $checks['recommendations'] = $this->generateProactiveRecommendations($checks);

    // Cache results for 5 minutes
    $this->cache->set($cache_key, $checks, time() + 300);

    return $checks;
  }

  /**
   * Gets recovery strategy for an exception.
   *
   * @param \Exception $exception
   *   The exception.
   *
   * @return string
   *   Recovery strategy identifier.
   */
  protected function getRecoveryStrategy(\Exception $exception) {
    $strategy_map = [
      DatabaseConnectionException::class => 'reconnect_database',
      MemoryExhaustedException::class => 'reduce_batch_size',
      VectorIndexCorruptedException::class => 'rebuild_vector_index',
      ApiKeyExpiredException::class => 'rotate_api_keys',
      EmbeddingServiceUnavailableException::class => 'restart_external_service',
      RateLimitException::class => 'enable_circuit_breaker',
      CacheDegradedException::class => 'clear_cache_and_retry',
    ];

    $exception_class = get_class($exception);
    return $strategy_map[$exception_class] ?? 'activate_fallback_mode';
  }

  /**
   * Attempts database reconnection with exponential backoff.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if reconnection successful.
   */
  protected function attemptDatabaseReconnection(array $context) {
    $max_attempts = 3;
    $base_delay = 1; // seconds

    for ($attempt = 1; $attempt <= $max_attempts; $attempt++) {
      try {
        // Test database connection
        $this->database->query('SELECT 1')->fetchField();
        
        $this->logger->info('Database reconnection successful on attempt {attempt}', [
          'attempt' => $attempt,
        ]);
        
        return TRUE;
      }
      catch (\Exception $e) {
        if ($attempt < $max_attempts) {
          $delay = $base_delay * pow(2, $attempt - 1);
          sleep($delay);
          
          $this->logger->warning('Database reconnection attempt {attempt} failed, retrying in {delay}s', [
            'attempt' => $attempt,
            'delay' => $delay,
            'error' => $e->getMessage(),
          ]);
        }
      }
    }

    $this->logger->error('Database reconnection failed after {attempts} attempts', [
      'attempts' => $max_attempts,
    ]);
    
    return FALSE;
  }

  /**
   * Queues vector index rebuild operation.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if rebuild was queued.
   */
  protected function queueVectorIndexRebuild(array $context) {
    try {
      $queue = $this->queueFactory->get('search_api_postgresql_vector_rebuild');
      
      $item = [
        'index_id' => $context['index_id'] ?? NULL,
        'priority' => 'high',
        'scheduled' => time(),
        'recovery_context' => $context,
      ];
      
      $queue->createItem($item);
      
      $this->logger->info('Vector index rebuild queued for {index_id}', [
        'index_id' => $context['index_id'] ?? 'unknown',
      ]);
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to queue vector index rebuild: {message}', [
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Clears relevant caches and retries operation.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if cache was cleared.
   */
  protected function clearCacheAndRetry(array $context) {
    try {
      // Clear specific cache bins
      $cache_bins = [
        'search_api_postgresql_embeddings',
        'search_api_postgresql_vectors',
        'search_api_postgresql_queries',
      ];
      
      foreach ($cache_bins as $bin) {
        if ($cache_backend = \Drupal::cache($bin)) {
          $cache_backend->deleteAll();
        }
      }
      
      // Clear general caches if specified
      if (!empty($context['clear_all_caches'])) {
        drupal_flush_all_caches();
      }
      
      $this->logger->info('Cache cleared for recovery operation');
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to clear cache: {message}', [
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Reduces batch size for memory-intensive operations.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if batch size was reduced.
   */
  protected function reduceBatchSize(array $context) {
    try {
      $current_batch_size = $context['batch_size'] ?? 1000;
      $new_batch_size = max(10, intval($current_batch_size * 0.5));
      
      // Store new batch size in configuration
      $config = $this->configFactory->getEditable('search_api_postgresql.settings');
      $config->set('recovery.reduced_batch_size', $new_batch_size);
      $config->set('recovery.batch_reduction_timestamp', time());
      $config->save();
      
      $this->logger->info('Batch size reduced from {old} to {new} for recovery', [
        'old' => $current_batch_size,
        'new' => $new_batch_size,
      ]);
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to reduce batch size: {message}', [
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Attempts to restart external service connections.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if restart was attempted.
   */
  protected function restartExternalService(array $context) {
    $service_name = $context['service_name'] ?? 'unknown';
    
    try {
      // Reset connection pools and caches for the service
      $this->resetServiceConnections($service_name);
      
      // Test service availability
      $is_available = $this->testServiceAvailability($service_name, $context);
      
      if ($is_available) {
        $this->logger->info('External service {service} restarted successfully', [
          'service' => $service_name,
        ]);
        return TRUE;
      }
      else {
        $this->logger->warning('External service {service} restart failed availability test', [
          'service' => $service_name,
        ]);
        return FALSE;
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to restart external service {service}: {message}', [
        'service' => $service_name,
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Rotates API keys for external services.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if keys were rotated.
   */
  protected function rotateApiKeys(array $context) {
    $service_name = $context['service_name'] ?? 'unknown';
    
    try {
      // This would integrate with key management system
      $this->logger->info('API key rotation requested for {service}', [
        'service' => $service_name,
      ]);
      
      // In a real implementation, this would:
      // 1. Request new API keys from the key management system
      // 2. Update configuration with new keys
      // 3. Test service connectivity with new keys
      // 4. Invalidate old keys
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to rotate API keys for {service}: {message}', [
        'service' => $service_name,
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Enables circuit breaker for a failing service.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if circuit breaker was enabled.
   */
  protected function enableCircuitBreaker(array $context) {
    $service_name = $context['service_name'] ?? 'unknown';
    
    try {
      $this->circuitBreakerStates[$service_name] = [
        'state' => 'open',
        'failure_count' => $context['failure_count'] ?? 0,
        'last_failure' => time(),
        'next_retry' => time() + 300, // 5 minutes
      ];
      
      $this->logger->info('Circuit breaker enabled for {service}', [
        'service' => $service_name,
      ]);
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to enable circuit breaker for {service}: {message}', [
        'service' => $service_name,
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Scales resources to handle increased load.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if scaling was initiated.
   */
  protected function scaleResources(array $context) {
    try {
      // This would integrate with infrastructure scaling systems
      $this->logger->info('Resource scaling requested due to recovery needs');
      
      // In a real implementation, this might:
      // 1. Increase memory limits
      // 2. Scale database connections
      // 3. Add more worker processes
      // 4. Request additional compute resources
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to scale resources: {message}', [
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Activates fallback mode for degraded functionality.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if fallback mode was activated.
   */
  protected function activateFallbackMode(array $context) {
    try {
      $config = $this->configFactory->getEditable('search_api_postgresql.settings');
      $config->set('recovery.fallback_mode', TRUE);
      $config->set('recovery.fallback_activated', time());
      $config->save();
      
      $this->logger->info('Fallback mode activated for enhanced resilience');
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to activate fallback mode: {message}', [
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Triggers emergency maintenance mode.
   *
   * @param array $context
   *   Recovery context.
   *
   * @return bool
   *   TRUE if maintenance mode was activated.
   */
  protected function triggerEmergencyMaintenance(array $context) {
    try {
      // Set site to maintenance mode
      \Drupal::state()->set('system.maintenance_mode', TRUE);
      
      $this->logger->critical('Emergency maintenance mode activated due to critical errors');
      
      return TRUE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to activate emergency maintenance: {message}', [
        'message' => $e->getMessage(),
      ]);
      
      return FALSE;
    }
  }

  /**
   * Checks database health and connectivity.
   *
   * @return array
   *   Database health status.
   */
  protected function checkDatabaseHealth() {
    try {
      $start_time = microtime(TRUE);
      $result = $this->database->query('SELECT 1')->fetchField();
      $response_time = (microtime(TRUE) - $start_time) * 1000; // ms
      
      $status = 'healthy';
      if ($response_time > 1000) {
        $status = 'warning';
      }
      if ($response_time > 5000) {
        $status = 'critical';
      }
      
      return [
        'status' => $status,
        'response_time' => $response_time,
        'connection_active' => TRUE,
        'message' => $status === 'healthy' ? 'Database responding normally' : 'Slow database response',
      ];
    }
    catch (\Exception $e) {
      return [
        'status' => 'critical',
        'response_time' => NULL,
        'connection_active' => FALSE,
        'message' => 'Database connection failed: ' . $e->getMessage(),
      ];
    }
  }

  /**
   * Checks memory usage and availability.
   *
   * @return array
   *   Memory health status.
   */
  protected function checkMemoryUsage() {
    $memory_usage = memory_get_usage(TRUE);
    $memory_limit = $this->parseMemoryLimit(ini_get('memory_limit'));
    $usage_percentage = $memory_limit > 0 ? ($memory_usage / $memory_limit) * 100 : 0;
    
    $status = 'healthy';
    if ($usage_percentage > 75) {
      $status = 'warning';
    }
    if ($usage_percentage > 90) {
      $status = 'critical';
    }
    
    return [
      'status' => $status,
      'usage_bytes' => $memory_usage,
      'limit_bytes' => $memory_limit,
      'usage_percentage' => $usage_percentage,
      'message' => sprintf('Memory usage: %.1f%% (%s / %s)', 
        $usage_percentage, 
        $this->formatBytes($memory_usage),
        $this->formatBytes($memory_limit)
      ),
    ];
  }

  /**
   * Checks disk space availability.
   *
   * @return array
   *   Disk space health status.
   */
  protected function checkDiskSpace() {
    try {
      $disk_free = disk_free_space('/');
      $disk_total = disk_total_space('/');
      $usage_percentage = $disk_total > 0 ? (($disk_total - $disk_free) / $disk_total) * 100 : 0;
      
      $status = 'healthy';
      if ($usage_percentage > 85) {
        $status = 'warning';
      }
      if ($usage_percentage > 95) {
        $status = 'critical';
      }
      
      return [
        'status' => $status,
        'free_bytes' => $disk_free,
        'total_bytes' => $disk_total,
        'usage_percentage' => $usage_percentage,
        'message' => sprintf('Disk usage: %.1f%% (%s free)', 
          $usage_percentage,
          $this->formatBytes($disk_free)
        ),
      ];
    }
    catch (\Exception $e) {
      return [
        'status' => 'unknown',
        'message' => 'Could not check disk space: ' . $e->getMessage(),
      ];
    }
  }

  /**
   * Checks vector index health.
   *
   * @return array
   *   Vector index health status.
   */
  protected function checkVectorIndexHealth() {
    // This would be implemented based on specific vector index requirements
    return [
      'status' => 'healthy',
      'indexes_checked' => 0,
      'corruption_detected' => FALSE,
      'message' => 'Vector index health check not implemented',
    ];
  }

  /**
   * Checks external service availability.
   *
   * @return array
   *   External services health status.
   */
  protected function checkExternalServices() {
    $services = [
      'azure_openai' => $this->checkAzureOpenAIHealth(),
      'embedding_cache' => $this->checkEmbeddingCacheHealth(),
    ];
    
    $overall_status = 'healthy';
    foreach ($services as $service_status) {
      if ($service_status['status'] === 'critical') {
        $overall_status = 'critical';
        break;
      }
      if ($service_status['status'] === 'warning' && $overall_status === 'healthy') {
        $overall_status = 'warning';
      }
    }
    
    return [
      'status' => $overall_status,
      'services' => $services,
      'message' => sprintf('%d external services checked', count($services)),
    ];
  }

  /**
   * Checks cache performance.
   *
   * @return array
   *   Cache performance status.
   */
  protected function checkCachePerformance() {
    try {
      $start_time = microtime(TRUE);
      $test_key = 'health_check_' . uniqid();
      $test_data = ['timestamp' => time()];
      
      // Test cache write
      $this->cache->set($test_key, $test_data);
      
      // Test cache read
      $cached = $this->cache->get($test_key);
      $response_time = (microtime(TRUE) - $start_time) * 1000; // ms
      
      // Clean up
      $this->cache->delete($test_key);
      
      $status = 'healthy';
      if ($response_time > 100) {
        $status = 'warning';
      }
      if ($response_time > 500 || !$cached) {
        $status = 'critical';
      }
      
      return [
        'status' => $status,
        'response_time' => $response_time,
        'read_successful' => (bool) $cached,
        'message' => sprintf('Cache response time: %.1fms', $response_time),
      ];
    }
    catch (\Exception $e) {
      return [
        'status' => 'critical',
        'message' => 'Cache health check failed: ' . $e->getMessage(),
      ];
    }
  }

  /**
   * Checks queue health and processing.
   *
   * @return array
   *   Queue health status.
   */
  protected function checkQueueHealth() {
    try {
      $queues = [
        'search_api_postgresql_embeddings',
        'search_api_postgresql_vector_rebuild',
      ];
      
      $queue_stats = [];
      $total_items = 0;
      
      foreach ($queues as $queue_name) {
        $queue = $this->queueFactory->get($queue_name);
        $items = $queue->numberOfItems();
        $queue_stats[$queue_name] = $items;
        $total_items += $items;
      }
      
      $status = 'healthy';
      if ($total_items > 1000) {
        $status = 'warning';
      }
      if ($total_items > 10000) {
        $status = 'critical';
      }
      
      return [
        'status' => $status,
        'total_items' => $total_items,
        'queue_breakdown' => $queue_stats,
        'message' => sprintf('%d items in queues', $total_items),
      ];
    }
    catch (\Exception $e) {
      return [
        'status' => 'unknown',
        'message' => 'Queue health check failed: ' . $e->getMessage(),
      ];
    }
  }

  /**
   * Checks API quotas and limits.
   *
   * @return array
   *   API quota status.
   */
  protected function checkApiQuotas() {
    // This would be implemented based on specific API requirements
    return [
      'status' => 'healthy',
      'quotas_checked' => 0,
      'near_limits' => [],
      'message' => 'API quota check not implemented',
    ];
  }

  /**
   * Helper method implementations for the service checks.
   */
  
  protected function checkAzureOpenAIHealth() {
    return ['status' => 'healthy', 'message' => 'Service available'];
  }
  
  protected function checkEmbeddingCacheHealth() {
    return ['status' => 'healthy', 'message' => 'Cache operational'];
  }
  
  protected function assessOverallHealth(array $checks) {
    $critical_count = 0;
    $warning_count = 0;
    
    foreach ($checks as $check) {
      if (isset($check['status'])) {
        if ($check['status'] === 'critical') {
          $critical_count++;
        } elseif ($check['status'] === 'warning') {
          $warning_count++;
        }
      }
    }
    
    if ($critical_count > 0) {
      return ['status' => 'critical', 'message' => "$critical_count critical issues detected"];
    }
    if ($warning_count > 0) {
      return ['status' => 'warning', 'message' => "$warning_count warnings detected"];
    }
    
    return ['status' => 'healthy', 'message' => 'All systems operational'];
  }
  
  protected function generateProactiveRecommendations(array $checks) {
    $recommendations = [];
    
    foreach ($checks as $check_name => $check) {
      if (isset($check['status']) && $check['status'] !== 'healthy') {
        switch ($check_name) {
          case 'memory_usage':
            if ($check['usage_percentage'] > 75) {
              $recommendations[] = 'Consider increasing memory allocation or optimizing memory usage';
            }
            break;
          case 'disk_space':
            if ($check['usage_percentage'] > 85) {
              $recommendations[] = 'Clean up temporary files and logs to free disk space';
            }
            break;
          case 'queue_health':
            if ($check['total_items'] > 1000) {
              $recommendations[] = 'Queue processing may be falling behind, consider adding workers';
            }
            break;
        }
      }
    }
    
    return $recommendations;
  }

  /**
   * Utility methods.
   */
  
  protected function generateRecoveryId(\Exception $exception, array $context) {
    return md5(get_class($exception) . serialize($context));
  }
  
  protected function hasExceededMaxAttempts($recovery_id) {
    $max_attempts = 5;
    $time_window = 3600; // 1 hour
    
    if (!isset($this->recoveryAttempts[$recovery_id])) {
      return FALSE;
    }
    
    $attempts = $this->recoveryAttempts[$recovery_id];
    $recent_attempts = array_filter($attempts, function($timestamp) use ($time_window) {
      return $timestamp > (time() - $time_window);
    });
    
    return count($recent_attempts) >= $max_attempts;
  }
  
  protected function trackRecoveryAttempt($recovery_id) {
    if (!isset($this->recoveryAttempts[$recovery_id])) {
      $this->recoveryAttempts[$recovery_id] = [];
    }
    $this->recoveryAttempts[$recovery_id][] = time();
  }
  
  protected function resetServiceConnections($service_name) {
    // Implementation would reset HTTP clients, connection pools, etc.
    $this->logger->info('Service connections reset for {service}', ['service' => $service_name]);
  }
  
  protected function testServiceAvailability($service_name, array $context) {
    // Implementation would test actual service connectivity
    return TRUE;
  }
  
  protected function parseMemoryLimit($limit_string) {
    $unit = strtolower(substr($limit_string, -1));
    $value = intval($limit_string);
    
    switch ($unit) {
      case 'g': return $value * 1024 * 1024 * 1024;
      case 'm': return $value * 1024 * 1024;
      case 'k': return $value * 1024;
      default: return $value;
    }
  }
  
  protected function formatBytes($bytes) {
    $units = ['B', 'KB', 'MB', 'GB'];
    $bytes = max($bytes, 0);
    $pow = floor(($bytes ? log($bytes) : 0) / log(1024));
    $pow = min($pow, count($units) - 1);
    
    return round($bytes / (1024 ** $pow), 2) . ' ' . $units[$pow];
  }
}